#pragma once

#include "CircuitElement.h"
#include "TruthTable.h"
#include "Wire.h"

#include "Pointer.h"
#include "Vector.h"

class LogicGate : public CircuitElement
{
public:
    enum class GateType
    {
        NOT,
        AND,
        OR,
        XOR,
        UNKNOWN
    };
    virtual ~LogicGate() = default;

    virtual size_t getNumInputs() const = 0;

    virtual size_t getNumOutputs() const = 0;

    virtual Wire* getInput(size_t idx) const = 0;

    virtual Wire* getOutput(size_t idx) const = 0;

    virtual const TruthTable& getTruthTable() = 0;

    virtual GateType getGateType() const = 0;

    Type getType() const override
    {
        return Type::LOGIC_GATE;
    }
};

class NInMOutLogicGate : public LogicGate
{
public:
    NInMOutLogicGate(size_t numIn, size_t numOut, Vector<Wire*> inputs = {}, Vector<Wire*> outputs = {});

    virtual ~NInMOutLogicGate() = default;

    size_t getNumInputs() const override;

    size_t getNumOutputs() const override;

    Wire* getInput(size_t idx) const override;

    Wire* getOutput(size_t idx) const override;

    void setAtInput(size_t idx, Wire* value);

    void setAtOutput(size_t idx, Wire* value);

private:
    size_t mNumIn{ 1 };
    size_t mNumOut{ 1 };

    Vector<Wire*> mInputs;
    Vector<Wire*> mOutputs;
};

class TwoInOneOutLogicGate : public NInMOutLogicGate
{
public:
    TwoInOneOutLogicGate(Wire* input0 = nullptr, Wire* input1 = nullptr, Wire* output = nullptr);

    void setInput0(Wire* input);

    void setInput1(Wire* input);

    void setOutput(Wire* output);
};