#pragma once

#include "BuildConfig.h"
#include "FileSystemPath.h"
#include "String.h"
#include "Vector.h"

#include "BuildEnvironment.h"

class BuildTarget
{
public:
    BuildTarget(BuildTargetType target_type);

    BuildTarget(Ptr<BuildConfig> config);

    static Ptr<BuildTarget> create(const FileSystemPath& build_config_path);

    virtual ~BuildTarget() = default;

    virtual Status build() = 0;

    void add_dependency(BuildTarget* dependency);

    virtual Status configure(const BuildEnvironment& env);

    BuildTargetType get_type() const;

    void get_public_include_dirs(Vector<FileSystemPath>& dirs) const;

    const String& get_name() const;

    const Vector<BuildTarget*>& get_dependencies() const;

    const Vector<String>& get_dependency_names() const;

protected:
    FileSystemPath get_directory() const;

    Vector<BuildTarget*> m_dependencies;

private:
    Status populate_include_dirs();

    Status load_config();

    Vector<String> m_dependency_names;
    Ptr<BuildConfig> m_build_config;
    Vector<FileSystemPath> m_include_dirs;
    String m_name;
    BuildTargetType m_type{BuildTargetType::UNSET};
};