#pragma once

#include "TemplateNode.h"

class TemplateExtends : public TemplateNode
{
public:
    TemplateExtends(TemplateNode* parent, const String& path);

    virtual ~TemplateExtends() = default;

    String getRawContent() const override;
    String getPath() const;

    Type getType() const override
    {
        return Type::EXTENDS;
    }

private:
    String mPath;
};

class TemplateBlock : public TemplateNode
{
public:
    TemplateBlock(TemplateNode* parent, const String& name);

    virtual ~TemplateBlock() = default;

    void addLine(const String& line);

    String getRawContent() const override;

    String getIdentifier() const override;

    Type getType() const override
    {
        return Type::BLOCK;
    }

    String renderAsParent(TemplateSubstitutionContext* substitutions, TemplateBlock* base);

    String renderAsLeaf(TemplateSubstitutionContext* substitutions);

    String render(TemplateSubstitutionContext* substitutions, TemplateNode* parentContext) override;

private:
    String mName;
    Vector<String> mBody;
};

class TemplateExpression : public TemplateNode
{
public:
    TemplateExpression(TemplateNode* parent, const String& content);

    virtual ~TemplateExpression() = default;

    String getRawContent() const override;

    const String& getContent() const;

    Type getType() const override
    {
        return Type::EXPRESSION;
    }

    String render(TemplateSubstitutionContext* substitutions, TemplateNode* parentContext) override;

private:
    String mContent;
};

class TemplateTextBody : public TemplateNode
{
public:
    TemplateTextBody(TemplateNode* parent);

    virtual ~TemplateTextBody() = default;

    void addLine(const String& content);

    String getRawContent() const override;

    bool hasContent() const;

    String render(TemplateSubstitutionContext* substitutions, TemplateNode* parentContext) override;

    Type getType() const override
    {
        return Type::TEXT_BODY;
    }
private:
    Vector<String> mContent;
};
