#pragma once

#include "Pointer.h"
#include "String.h"
#include "Vector.h"

class TemplateSubstitutionContext;

class TemplateNode
{
public:
    enum class Type
    {
        NONE,
        BLOCK,
        EXPRESSION,
        EXTENDS,
        TEXT_BODY
    };

    TemplateNode(TemplateNode* parent);

    virtual ~TemplateNode() = default;

    virtual void addChild(Ptr<TemplateNode> child);

    TemplateNode* getChild(size_t index) const;

    TemplateNode* getFirstChildShallow(Type type, const String& identifier = {}) const;

    virtual String getIdentifier() const;

    size_t getNumChildren() const;

    TemplateNode* getParent() const;

    virtual String getRawContent() const;

    virtual Type getType() const;

    virtual String render(TemplateSubstitutionContext* substitutions, TemplateNode* parentContext = nullptr);

    void setExtensionParent(TemplateNode* parent);

    void setExtensionBase(TemplateNode* base);

protected:
    Vector<Ptr<TemplateNode> > mChildren;
    TemplateNode* mParent{ nullptr };
    TemplateNode* mExtensionParent{ nullptr };
    TemplateNode* mExtensionBase{ nullptr };
};

using TemplateNodePtr = Ptr<TemplateNode>;
