#pragma once

#include "RawTree.h"

#include "HuffmanCodeLengthTable.h"
#include "HuffmanFixedCodes.h"

#include "Vector.h"
#include <tuple>
#include Map.h

class PrefixCodeGenerator
{
public:
    virtual ~PrefixCodeGenerator() = default;
    virtual Optional<PrefixCode> getLiteralValue(unsigned char symbol) const = 0;
    virtual Optional<PrefixCode> getLengthValue(unsigned length) const = 0;
    virtual Optional<PrefixCode> getDistanceValue(unsigned distance) const = 0;

    virtual Optional<PrefixCode> getEndOfStreamValue() const = 0;
};

class HuffmanEncoder : public PrefixCodeGenerator
{
    using CountPair = std::pair<unsigned, unsigned>;
    using Hit = std::tuple<unsigned, unsigned, unsigned char>;

public:
    void encode(const Vector<unsigned>& counts);
    void encode(const Map<unsigned char, unsigned>& counts);

    uint32_t getLengthValue(unsigned length);

    Optional<PrefixCode> getLiteralValue(unsigned char symbol) const override;

    Optional<PrefixCode> getLengthValue(unsigned length) const override;

    Optional<PrefixCode> getDistanceValue(unsigned distance) const override;

    Optional<PrefixCode> getEndOfStreamValue() const override;

    void initializeTrees(const Vector<Hit>& hits);

    void setUseFixedCode(bool useFixed);
private:
    void initializeLiteralLengthTable(const Vector<Hit>& hits);

    void dumpTree(const RawTree<CountPair>& tree) const;
    void dumpNode(RawNode<CountPair>* node, unsigned depth) const;

    bool mUseFixedCode{false};

    Vector<unsigned char> mSymbolMapping;
    HuffmanCodeLengthTable mLiteralLengthTable;
    HuffmanCodeLengthTable mDistanceTable;
};
