#pragma once

#include "Vector.h"
#include "String.h"
#include <optional>
#include <cstdint>

class PrefixCode
{
public:
    PrefixCode(uint32_t data, unsigned length);

    String toString(bool bitsAsRightToLeft = true) const;

    bool matches(unsigned length, uint32_t code) const;

    uint32_t getData() const
    {
        return mData;
    }

    unsigned getLength() const
    {
        return mLength;
    }

private:
    unsigned mLength{0};
    uint32_t mData{0};
};

class HuffmanTree
{
public:
    using Symbol = unsigned;
    using CodeLength = unsigned;
    using CodeSymbolPair = std::pair<PrefixCode, Symbol>;
    using CodeLengthData = std::pair<CodeLength, Vector<CodeSymbolPair> >;

    void addCodeLengthEntry(unsigned length, const CodeSymbolPair& data);

    String dump(bool bitsAsRightToLeft = true) const;

    Optional<HuffmanTree::Symbol> findMatch(size_t treeIndex, uint32_t code) const;

    size_t getNumCodeLengths() const;

    unsigned getCodeLength(size_t idx) const;

    Optional<PrefixCode> getCode(Symbol symbol) const;

    void sortTable();
private:
    Vector<CodeLengthData> mTable;
};
