#pragma once

#include "Pointer.h"
#include "Vector.h"
#include "String.h"

class Color
{
public:
    enum class Format 
    {
        GRAYSCALE,
        GRAYSCALE_ALPHA,
        LUT,
        RGB,
        ARGB,
        RGBA
    };

    Color(const String& hexString);
    Color(unsigned char r = 0, unsigned char g = 0, unsigned char b = 0, double a = 1.0);

    static Ptr<Color> Create(unsigned char r, unsigned char g, unsigned char b, double a = 1.0);
    static Ptr<Color> Create(const Color& color);

    unsigned char getR() const;
    unsigned char getG() const;
    unsigned char getB() const;
    double getAlpha() const;

    Vector<double> getAsVectorDouble() const;

    uint32_t getAsUInt32() const;

    String toString() const;

    size_t getSize() const;

    static size_t getNumChannels(Format format);

    static size_t getSize(Format format, unsigned bitDepth);

    uint8_t getByte(size_t index) const;

    void setAlpha(float alpha);

    bool operator==(const Color& rhs) const
    {
       return (mR == rhs.mR)
       && (mG == rhs.mG)
       && (mB == rhs.mB)
       && (mAlpha == rhs.mAlpha);
    }

    bool operator!=(const Color& rhs) const
    {
      return !operator==(rhs);
    }

private:
    unsigned char toDecimal(const String& hex) const;

    unsigned char mR{0};
    unsigned char mG{0};
    unsigned char mB{0};
    unsigned char mValue{0};
    double mAlpha{0.0};
    unsigned mBitDepth{8};
    Format mFormat{Format::RGBA};
};
