#include "Error.h"

#include <string.h>
#include <errno.h>

Error::Error(const String& msg)
        : m_message(msg)
{
}

const String& Error::msg() const 
{
    return m_message;
}

String Error::from_errno()
{
    return ::strerror(errno);
}

Status::Status(const Error& err)
    : m_error(err),
        m_ok(false)
{

}

Status Status::with_errno(const String& prefix_msg)
{
    Status ret;
    ret.on_errno(prefix_msg);
    return ret;
}

void Status::on_errno(const String& prefix_msg)
{
    String errno_msg(::strerror(errno));
    m_error = Error(prefix_msg + " | " + errno_msg);
    m_ok = false;
}

const Error& Status::error() const 
{
    return m_error;
}

bool Status::ok() const 
{
    return m_ok;
}

String Status::str() const 
{
    if (!m_ok)
    {
        return m_error.msg();
    }
    else 
    {
        return {};
    }
}