#pragma once

#include "String.h"

#define IF_OK_AND_TRUE(PRED) \
    const auto _result = PRED; \
    if (!_result.ok()){return Status(_result.error());} \
    if (_result.value())

#define ERROR_IF_NOT_OK_OR_TRUE(PRED, msg) \
    {const auto _result = PRED; \
    if (!_result.ok()){return {_result.error()};} \
    if (!_result.value()){return {Error(msg)};}}

#define ON_ERRNO(msg) \
    {Status _status; \
    _status.on_errno("Failed to get directory content in opendir"); \
    return _status;} \

#define ON_ERRNO_RESULT(msg) \
    return {Error(_s(msg) + " | " + Error::from_errno())};

#define STATUS_CHECK(PRED, error_msg) \
    {const auto _status = PRED; \
    if (!_status.ok()){ \
        const auto message = _s(error_msg) + "|" + _status.error().msg(); \
        return Status(Error(message)); \
    }}

class Error
{
public:
    Error(const String& msg);

    Error() = default;

    const String& msg() const;

    static String from_errno();
private:
    String m_message;
};

class Status
{
public:
    Status() = default;

    Status(const Error& err);

    const Error& error() const;

    static Status with_errno(const String& prefix_msg);

    void on_errno(const String& prefix_msg);

    bool ok() const;

    String str() const;

private:
    Error m_error;
    bool m_ok{true};
};