#pragma once

#include "Vector.h"

template<typename T>
class CircleBuffer
{
public:
    CircleBuffer(size_t size)
        : mData(size)
    {

    }

    void addItem(const T& item)
    {
        if (mEndPointer < mData.size())
        {
            mData[mEndPointer] = item;
            mEndPointer++;
        }
        else
        {
            mData[mStartPointer] = item;
            if (mStartPointer < mData.size() - 1)
            {
                mStartPointer++;
            }
            else
            {
                mStartPointer = 0;
            }
        }
    }

    size_t getNumItems() const
    {
        return mEndPointer;
    }

    const T& getItem(size_t index) const
    {
        if (mEndPointer < mData.size())
        {
            return mData[index];
        }
        else
        {
            auto offset = mStartPointer + index;
            if (offset >= mData.size())
            {
                offset -= mData.size();
            }
            return mData[offset];
        }
    }

private:
    size_t mStartPointer{0};
    size_t mEndPointer{0};
    Vector<T> mData;
};
