#pragma once

#include "String.h"
#include "Number.h"
#include "Vector.h"
#include "Map.h"
#include "Pointer.h"
#include "Optional.h"

class Dictionary
{
public:
    enum class Type
    {
        MAP,
        SEQUENCE,
        SCALAR
    };

    enum class ScalarType 
    {
        STRING,
        NUMBER,
        BOOLEAN,
        NULLT
    };

    Dictionary() = default;
    Dictionary(const String& item);

    virtual ~Dictionary() = default;

    void addItem(const String& key, const String& item);

    void addItem(const String& key, Ptr<Dictionary> dict);

    Dictionary* getItemAsMap(const String& key) const;

    Optional<String> getItemAsString(const String& key) const;

    Vector<String> getKeys() const;

    bool hasKey(const String& key) const;

    bool isMap() const;

    bool isString() const;
    
protected:
    Type mType{Type::MAP};
    ScalarType mScalarType{ScalarType::STRING};
    String mStringVal;
    bool mBoolVal{false};
    Number mNumberVal;
    Vector<Ptr<Dictionary>> mSequenceVal;
    Map<String, Ptr<Dictionary> > mMapVal;
};
