#pragma once

#include "Vector.h"
#include "Pair.h"
#include "Index.h"
#include "Byte.h"

#include <functional>

class String
{
public:
    String();

    String(const Vector<Byte>& data);

    String(const char* data);

    String(size_t size, char c);

    static String fmt(const char* fmt, ...);

    void append(const Vector<Byte>& data);

    bool contains(const String& term) const;

    const Vector<char>& data() const;

    bool empty() const;

    using erasePredicate = std::function<bool(char)>;
    void eraseIf(erasePredicate func);

    static size_t length(const char* arr);

    bool is_whitespace() const;

    void push_back(char c);

    const char* raw() const;

    Pair<String, String> rsplit(char c) const;

    Index rindex(char c) const;

    void reverse();

    size_t size() const;

    bool slice(size_t idx, String& out) const;

    bool slice(size_t start, size_t end, String& out) const;

    void split(Vector<String>& output, char delimiter = ' ') const;

    bool starts_with(const String& prefix) const;

    static String to_string(size_t input);

    void to_bytes(Vector<Byte>& bytes) const;

    void to_lower();

    static String to_lower(const String& input);

    char operator[](size_t idx) const;

    String& operator<<(const String& body);

    String& operator<<(const char* body);

    String& operator<<(size_t idx); 

    String& operator<<(const int idx); 

    template<typename T>
    String& operator<<(const T& stringable)
    {
        *this += stringable.str();
        return *this;
    } 

    String& operator+=(const String& str);

    String& operator+=(char c);

    String operator+(const String& str) const;

    bool operator==(const String& other) const;

    bool operator!=(const String& other) const;

    class StringIter
    {
        public:
        StringIter(const String* container, int index = -1)
            : m_index(index), m_container(container)
        {
        }

        bool operator!=(const StringIter& other) const 
        {
            return m_index != other.m_index;
        }

        char operator*() const 
        {
            return (*m_container)[m_index];
        }

        void operator++()
        {
            m_index++;
        }

        private:
        int m_index{-1};
        const String* m_container{nullptr};
    };

    StringIter begin() const
    {
        if (empty())
        {
            return StringIter(this, -1);
        }
        else
        {
            return StringIter(this, 0);
        }
    }

    StringIter end() const
    {
        if (empty())
        {
            return StringIter(this, -1);
        }
        else
        {
            return StringIter(this, size());
        }
    }

private:
    void append(const char* data);

    Vector<char> m_data;
};

inline String operator+(const char* str, const String& s)
{
    String ret(str);
    ret += s;
    return ret;
}

using _s = String;