#include "SharedMemory.h"

#include "RandomUtils.h"

#ifdef __linux__
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#endif

void SharedMemory::allocate(const String& namePrefix, size_t size)
{
    createFile(namePrefix);

    if (!mIsValid)
    {
        return;
    }

#ifdef __linux__
    int ret{-1};
    do {
        ret = ftruncate(mFileDescriptor, size);
    } while (ret < 0 && errno == EINTR);

    if (ret < 0)
    {
        close(mFileDescriptor);
        mIsValid = false;
    }
#else
    (void)(size);
#endif
}

int SharedMemory::getFileDescriptor() const
{
    return mFileDescriptor;
}

bool SharedMemory::isValid() const
{
    return mIsValid;
}

void SharedMemory::createFile(const String& namePrefix)
{
#ifdef __linux__
    unsigned retries = 100;
    do {
        const auto name = getRandomName(namePrefix);
        --retries;

        const int fd = shm_open(name.raw(), O_RDWR | O_CREAT | O_EXCL, 0600);
        if (fd >= 0)
        {
            shm_unlink(name.raw());
            mFileDescriptor = fd;
            mIsValid = true;
            break;
        }
    } while (retries > 0 && errno == EEXIST);
#else
    (void)(namePrefix);
#endif 
}

String SharedMemory::getRandomName(const String& namePrefix) const
{
    String randomSuffix;
    for (const auto entry : RandomUtils::getRandomVecUnsigned(6))
    {
        randomSuffix += String::to_string(entry);
    }
    return namePrefix + randomSuffix;
}
