#include "HttpRequest.h"

#include "StringUtils.h"
#include "HttpParser.h"

HttpRequest::HttpRequest(Verb verb, const String& path)
    :   mVerb(verb)
{
    mPreamble.mPath = path;
}

HttpRequest::Verb HttpRequest::getVerb() const
{
    return mVerb;
}

String HttpRequest::getPath() const
{
    return mPreamble.mPath;
}

String HttpRequest::toString(const String& host) const
{
    String out;

    if (mVerb == Verb::GET)
    {
        out += "GET";
    }

    auto path = mPreamble.mPath;
    out += _s(" /") + path + _s(" HTTP/") + mHeader.getHttpVersion() + _s("\n");
    out += _s("Host: ") + host + _s("\n");
    out += "Accept - Encoding: \n";
    return out;
}

void HttpRequest::fromString(const String& message)
{
    String buffer;
    bool firstLine{ true };

    Vector<String> headers;
    Vector<String> lines;
    message.split(lines, '\n');
    for(const auto& line : lines)
    {
        if (firstLine)
        {
            HttpParser::parsePreamble(line, mPreamble);
            firstLine = false;
        }
        else
        {
            headers.push_back(line);
        }
    }
    if (mPreamble.mMethod == "GET")
    {
        mVerb = Verb::GET;
    }
    mHeader.parse(headers);

    mRequiredBytes = 0;
}

size_t HttpRequest::requiredBytes() const
{
    return mRequiredBytes;
}
