#include "HttpResponse.h"

#include "StringUtils.h"
#include "HttpParser.h"

HttpResponse::HttpResponse()
    :   mStatusCode(200),
        mResponseReason("OK"),
        mBody()
{

}

HttpResponse::~HttpResponse()
{

}

const HttpHeader& HttpResponse::getHeader() const
{
    return mHeader;
}

unsigned short HttpResponse::getStatusCode() const
{
    return mStatusCode;
}

const String& HttpResponse::getBody() const
{
    return mBody;
}

void HttpResponse::setBody(const String& body)
{
    mBody = body;
}

void HttpResponse::fromMessage(const String& message)
{
    String buffer;
    bool firstLine{ true };

    Vector<String> headers;
    Vector<String> lines;
    message.split(lines, '\n');
    for(const auto& line : lines)
    {
        if (firstLine)
        {
            HttpParser::parsePreamble(line, mPreamble);
            firstLine = false;
        }
        else
        {
            headers.push_back(line);
        }
    }
    mHeader.parse(headers);
}

unsigned HttpResponse::getBodyLength() const
{
    return unsigned(mBody.size());
}

void HttpResponse::setClientError(const ClientError& error)
{
    mClientError = error;
}

String HttpResponse::getHeaderString() const
{
    String header = _s("HTTP/") + mHeader.getHttpVersion() + _s(" ") + String::to_string(mStatusCode) + _s(" ") + mResponseReason + _s("\n");
    header += _s("Content-Type: ") + mHeader.getContentType() + _s("\n");
    header += _s("Content-Length: ") + String::to_string(getBodyLength()) + _s("\n");
    return header;
}

const String& HttpResponse::getResponseReason() const
{
    return mResponseReason;
}

String HttpResponse::toString() const
{
    return getHeaderString() + _s("\n\n") + mBody;
}

void HttpResponse::setStatusCode(unsigned short code)
{
    mStatusCode = code;
}

void HttpResponse::setResponseReason(const String& reason)
{
    mResponseReason = reason;
}
