#pragma once

#include "File.h"
#include "FileSystemPath.h"
#include "Pointer.h"
#include "Map.h"
#include "String.h"

class TomlTable
{
public:
    using Comment = Pair<unsigned, String>;
    using KeyValuePairs = Map<String, String>;

    TomlTable(const String& header);

    void addComment(const Comment& comment);

    void addTable(Ptr<TomlTable> table);

    void addKeyValuePair(const String& key, const String& value);

    String getHeader() const;

    TomlTable* getTable(const String& path) const;

    const KeyValuePairs& getKeyValuePairs() const;

private:
    String mHeader;
    Map<String, Ptr<TomlTable> > mTables;
    KeyValuePairs mMap;
    Vector<Comment> mComments;
};

class TomlContent
{
public:
    TomlContent();

    TomlTable* getRootTable() const;

    TomlTable* getTable(const String& path) const;

private:
    Ptr<TomlTable> mRootTable;
};

class TomlReader
{
public:
    TomlReader();

    TomlContent* getContent() const;

    void read(const FileSystemPath& input_path);

    void processLine(const String& line);

    void onHeader(const String& header);

    void onKeyValuePair(const String key, const String value);

private:
    unsigned mLastSectionOffset{ 0 };
    Ptr<TomlContent> mContent;
    TomlTable* mWorkingTable{nullptr};
};
