#include "Process.h"

#include "Logger.h"
#include "File.h"

#include <unistd.h>
#include <sys/wait.h>
#include <errno.h>

Status Process::launch(const String& command, 
    const Vector<String>& args)
{
    const auto pid = fork();
    if (pid < 0)
    {
        ON_ERRNO("Failed to fork");
    }

    if (pid == 0)
    {
        char* exe_args[args.size()+2];
        exe_args[0] = const_cast<char*>(command.raw());
        for(size_t idx = 1; idx<args.size()+1; idx++)
        {
            //LOG_INFO(args[idx-1]);
            exe_args[idx] = const_cast<char*>(args[idx-1].raw());
        }
        exe_args[args.size()+1] = nullptr;
        errno = 0;
        auto rc = execv(command.raw(), exe_args);
        if (rc != 0)
        {
            LOG_ERROR("External proc failed: " << Error::from_errno());
            exit(EXIT_FAILURE);
        }
        exit(EXIT_SUCCESS);
    }
    else
    {
        int status;
        auto rc = wait(&status);
        if (rc < 0)
        {
            ON_ERRNO("Failed to wait");
        }
        if (!WIFEXITED(status))
        {
            return Status(Error("Process child did not termintate normally"));
        }
        if (WEXITSTATUS(status) != EXIT_SUCCESS)
        {
            return Status(Error("Process child terminated with failure"));
        }
    }
    return {};
}

Result<String> Process::get_self_name()
{
    FileSystemPath path(String("/proc/self/cmdline"));
    File sys_file(path);

    Result<String> ret;
    const auto rc = sys_file.read(ret.value());
    if (!rc.ok())
    {
        ret.on_error(rc.error());
    }
    return ret;
}