#pragma once

#include "List.h"
#include "AbstractGrid.h"

#include "Pointer.h"

template <typename T>
class TypedGrid : public AbstractGrid
{
public:
    TypedGrid(const Bounds& bounds, size_t numPointsX, size_t numPointsY, size_t numPointsZ = 1)
        : AbstractGrid(bounds, numPointsX, numPointsY, numPointsZ),
            mData(std::make_unique<List<T> >())
    {
    }

    virtual ~TypedGrid() = default;

    virtual T getItem(size_t idx, size_t jdx, size_t kdx) const = 0;

    virtual void setItem(size_t idx, size_t jdx, size_t kdx, const T& value) = 0;

    List<T>* getInternalData() const
    {
        return mData.get();
    }
protected:
    Ptr<List<T> > mData;
};
