#pragma once

#include "Pointer.h"
#include <filesystem>

#include "AbstractApp.h"
#include "AudioManager.h"
#include "FileLogger.h"
#include "DatabaseManager.h"
#include "NetworkManager.h"
#include "CommandLineArgs.h"

class MainApplication : public AbstractApp
{
public:
    MainApplication();

    ~MainApplication();

    static Ptr<MainApplication> Create();

    void initialize(CommandLineArgsUPtr commandLineArgs, Ptr<IApplicationContext> applicationContext=nullptr);

    void run();

    void shutDown();

    CommandLineArgs* getCommandLineArgs() const;

private:
    void runServer();

    void playAudio();

    void convertDocument(const String& inputPath, const String& outputPath);

    CommandLineArgsUPtr mCommandLineArgs;
    DatabaseManagerPtr mDatabaseManager;
    NetworkManagerUPtr mNetworkManager;
    AudioManagerUPtr mAudioManager;
};

using MainApplicationPtr = std::shared_ptr<MainApplication>;
