#pragma once

#include "String.h"
#include "Pointer.h"
#include "Vector.h"
#include "File.h"
#include "IFont.h"

class File;

class FontReader
{
public:
    ~FontReader();
    void setPath(const String& path);

    Ptr<IFont> read();

private:

    struct OffsetSubtable
    {
        uint32_t scaler_type{0};
        uint16_t num_tables{0};
        uint16_t search_range{0};
        uint16_t entry_selector{0};
        uint16_t range_shift{0};
    };

    struct Table
    {
        String name;
        unsigned checksum{0};
        unsigned offset{0};
        unsigned length{0};
    };

    bool readOffsetSubtable();
    void logOffsetSubtable();
    void logTable(const Table& table);

    void readTableDirectory();

    void readTable();
    void readHeadTable();

    unsigned mCurrentOffset{0};
    OffsetSubtable mOffsetSubtable;
    Vector<Table> mTables;

    Ptr<IFont> mWorkingFont;
    Ptr<File> mFile;
    String mPath;
};
