#pragma once

#include "Point.h"
#include "Transform.h"

#include Map.h
#include "String.h"
#include "Vector.h"

class Node
{
public:

    enum class State
    {
        HEALTHY,
        DIRTY
    };

    Node(const Point3& p, size_t index = 0);

    static Ptr<Node> Create(const Point3& p, size_t index = 0);

    ~Node();

    void apply(const Transform& transform);

    void associateEdge(size_t edgeId);

    void associateFace(size_t faceId);

    void addVectorAttribute(const String& tag, const Vector<double>& values);

    void clearConnectivity();

    size_t getIndex() const;

    size_t getNumConnectedEdges() const;

    size_t getNumConnectedFaces() const;

    size_t getConnectedEdgeId(size_t idx) const;

    size_t getConnectedFaceId(size_t idx) const;

    State getState() const;

    const Point3& getPoint() const;

    Vector<double> getVectorAttribute(const String& tag) const;

    bool isCoincident(Node* node) const;

    void setState(State state);

    void setIndex(size_t idx);

    void updateIndex(size_t index);

private:
    Map<String, Vector<double> > mVectorAttributes;
    size_t mIndex{0};
    Point3 mPoint;

    Vector<size_t> mAssociatedEdgeIds;
    Vector<size_t> mAssociatedFaceIds;

    State mState{State::HEALTHY};
};
