#pragma once

#include "SceneItem.h"
#include "AbstractGeometricItem.h"
#include "Texture.h"

#include "Vector.h"
#include "String.h"
#include Map.h
#include "Pointer.h"

class AbstractMesh;

class SceneModel : public SceneItem
{
public:
    SceneModel() = default;
    virtual ~SceneModel();

    explicit SceneModel(Ptr<AbstractMesh> mesh);
    explicit SceneModel(Ptr<AbstractGeometricItem> geometry);

    AbstractGeometricItem* getGeometry() const;
    AbstractMesh* getMesh() const;
    bool getShowOutline() const;
    Type getType() const override;

    void setMeshTransform(const Transform& transform);
    void setShowOutline(bool showOutline);

    void updateGeometry(Ptr<AbstractGeometricItem> geometry);
    void updateMesh(Ptr<AbstractMesh> mesh);
    void updateMesh(AbstractMesh* mesh);

private:
    AbstractMesh* mRawMesh{nullptr};
    Ptr<AbstractMesh> mMesh;
    Transform mMeshTransform;

    Ptr<Texture> mColorMap;

    Ptr<AbstractGeometricItem> mGeometry;

    bool mGeometryIsDirty{true};
    bool mShowOutline{false};
};
